#!/usr/bin/env python

import argparse
import pandas as pd
import os

from .metadata import *
from .docker import *

def main():
	parser = argparse.ArgumentParser()
	parser.add_argument("-g", "--gradescope", action="store_true", default=False,)
	parser.add_argument("-c", "--canvas", action="store_true", default=False)
	parser.add_argument("-j", "--json", default=False)
	parser.add_argument("-y", "--yaml", default=False)
	parser.add_argument("-n", "--notebooks-path", dest="notebooks-path", type=str, default="./")
	parser.add_argument("-t", "--tests-path", dest="tests-path", type=str, default="./tests/")
	parser.add_argument("-o", "--output-path", dest="output-path", type=str, default="./")
	parser.add_argument("-v", "--verbose", action="store_true")
	parser.add_argument("-r", "--requirements", type=str)
	parser.add_argument("--pdf", action="store_true", default=False)
	params = vars(parser.parse_args())

	# Asserts that exactly one metadata flag is provided
	assert sum([meta != False for meta in [params["gradescope"], 
		params["canvas"], 
		params["json"], 
		params["yaml"]]]) == 1, "You must supply exactly one metadata flag (-g, -j, -y, -c)"

	# verbose flag
	verbose = params["verbose"]

	# Hand off metadata to parser
	if params["gradescope"]:
		meta_parser = GradescopeParser(params["notebooks-path"])
		if verbose:
			print("Found Gradescope metadata...")
	elif params["canvas"]:
		meta_parser = CanvasParser(params["notebooks-path"])
		if verbose:
			print("Found Canvas metadata...")
	elif params["json"]:
		meta_parser = JSONParser(os.path.join(params["notebooks-path"], params["json"]))
		if verbose:
			print("Found JSON metadata...")
	else:
		meta_parser = YAMLParser(os.path.join(params["notebooks-path"], params["yaml"]))
		if verbose:
			print("Found YAML metadata...")

	# check that reqs file is valid
	if params["requirements"]:
		assert os.path.exists(params["requirements"]) and \
			os.path.isfile(params["requirements"]), "Requirements file '{}' does not exist.".format(params["requirements"])

	if verbose:
		print("Launching docker containers...")

	# Docker
	grades_df = grade_assignments(params["tests-path"], 
		params["notebooks-path"], 
		"42", 
		verbose=verbose, 
		pdfs=params["pdf"], 
		reqs=params["requirements"]
	)

	if verbose:
		print("Combining grades and saving...")

	# Merge Dataframes
	output_df = merge_export_csv([grades_df], params["output-path"])

	def map_files_to_ids(row):
		"""Returns the identifier for the filename in the specified row"""
		return meta_parser.file_to_id(row["file"])

	# add in identifier column
	output_df["identifier"] = output_df.apply(map_files_to_ids, axis=1)

	# write to CSV file
	output_df.to_csv(os.path.join(params["output-path"], "final_grades.csv"), index=False)


# Exports a list of dataframes as a single, merged csv file
def merge_export_csv(dataframes, output_path):
	original_directory = os.getcwd()
	os.chdir(output_path)
	final_dataframe = pd.concat(dataframes, axis=1, join='inner').sort_index()
	os.chdir(original_directory)
	return final_dataframe



import os

def list_files(path):
	"""Returns a list of all non-hidden files in a directory"""
	return [file for file in os.listdir(path) if os.path.isfile(os.path.join(path, file)) and file[0] != "."]


#!/usr/bin/env python

import argparse
import os
from os.path import isfile, join
from glob import glob
from gofer.ok import grade_notebook
import pandas as pd
import nb2pdf
import re

def grade(ipynb_path, pdf):
	base_path = os.path.dirname(ipynb_path)
	test_files = glob('/home/tests/*.py')
	result = grade_notebook(ipynb_path, test_files)
	if pdf:
		nb2pdf.convert(ipynb_path)
	return result

def main():
	argparser = argparse.ArgumentParser()
	argparser.add_argument('notebook_directory', help='Path to directory with ipynb\'s to grade')
	argparser.add_argument("--pdf", action="store_true", default=False)
	args = argparser.parse_args()
	dir_path = os.path.abspath(args.notebook_directory)
	os.chdir(os.path.dirname(dir_path))
	all_ipynb = [(f, join(dir_path, f)) for f in os.listdir(dir_path) if isfile(join(dir_path, f)) and f[-6:] == ".ipynb"]

	all_results = {"file": [], "score": [], "manual": []}
	
	if not args.pdf:
		del all_results["manual"]
	
	for ipynb_name, ipynb_path in all_ipynb:
		all_results["file"].append(ipynb_name)
		score = grade(ipynb_path, args.pdf)
		all_results["score"].append(score)
		if args.pdf:
			pdf_path = re.sub(r"\.ipynb$", ".pdf", ipynb_path)
			all_results["manual"].append(pdf_path)

	final_results = pd.DataFrame(all_results)
	final_results.to_csv("grades.csv", index=False)

if __name__ == "__main__":
	main()


import unittest
from local_grader.metadata import GradescopeParser, CanvasParser, JSONParser, YAMLParser

class TestGradescopeParser(unittest.TestCase):
	def setUp(self):
		self._path = "test/sample-metadata/gradescope-export"
		self._correct_result = [{
			"identifier": "12345",
			"filename": "file1.pdf"
		}, {
			"identifier": "23456",
			"filename": "file2.pdf"
		}, {
			"identifier": "someName",
			"filename": "odd_filename--s2034.ipynb"
		}, {
			"identifier": "someName",
			"filename": "GROUPodd_filename--s2034.ipynb"
		}, {
			"identifier": "someOtherName",
			"filename": "GROUPodd_filename--s2034.ipynb"
		}, {
			"identifier": "someOtherOtherName",
			"filename": "GROUPodd_filename--s2034.ipynb"
		}]

	def test_metadata(self):
		parser = GradescopeParser(self._path)
		self.assertEqual(parser.get_metadata(), self._correct_result)

class TestCanvasParser(unittest.TestCase):
	def setUp(self):
		self._path = "test/sample-metadata/canvas-export"
		self._correct_result = [{
			"identifier": "odd",
			"filename": "odd_filename--s2034.ipynb"
		}, {
			"identifier": "johndoe",
			"filename": "johndoe_LATE_5362839_74575800_markdown-intro (1).ipynb"
		}]

	def test_metadata(self):
		parser = CanvasParser(self._path)
		self.assertEqual(parser.get_metadata(), self._correct_result)

class TestJSONParser(unittest.TestCase):
	def setUp(self):
		self._path = "test/sample-metadata/test-metadata.json"
		self._correct_result = [{
			"identifier": "12345",
			"filename": "file1.pdf"
		}, {
			"identifier": "23456",
			"filename": "file2.pdf"
		}, {
			"identifier": "someName",
			"filename": "odd_filename--s2034.ipynb"
		}]

	def test_metadata(self):
		parser = JSONParser(self._path)
		self.assertEqual(parser.get_metadata(), self._correct_result)

class TestYAMLParser(unittest.TestCase):
	def setUp(self):
		self._path = "test/sample-metadata/test-metadata.yml"
		self._correct_result = [{
			"identifier": "12345",
			"filename": "file1.pdf"
		}, {
			"identifier": "23456",
			"filename": "file2.pdf"
		}, {
			"identifier": "someName",
			"filename": "odd_filename--s2034.ipynb"
		}]

	def test_metadata(self):
		parser = YAMLParser(self._path)
		self.assertEqual(parser.get_metadata(), self._correct_result)

if __name__ == "__main__":
	unittest.main()


############################################
##### Metadata Parser for Local Grader #####
############################################

import yaml
import json
import glob
import os
import re

from .utils import *

class GradescopeParser:
	"""Metadata parser for Gradescope exports"""

	def __init__(self, submissions_dir):
		# open metadata file and load into Python object
		with open(os.path.join(submissions_dir, "submission_metadata.yml")) as f:
			metadata = yaml.safe_load(f)

		# initialize metadata list
		self._metadata = []
		for file in metadata:
			# iterate through submitters for group submissions
			for submitter in metadata[file][":submitters"]:
				self._metadata += [{

					# metadata is separated by filename into a list of submitters
					# and we use the SID as the identifier
					"identifier": submitter[":sid"],
					"filename": file
				}]

		# TODO: handle group subs in _file_to_id because second sub will overwrite first
		self._file_to_id = {file["filename"] : file["identifier"] for file in self._metadata}
		self._id_to_file = {file["identifier"] : file["filename"] for file in self._metadata}

	def get_metadata(self):
		"""Returns mapping of identifiers to files"""
		return self._metadata

	def file_to_id(self, file):
		"""Returns a identifier given a filename"""
		return self._file_to_id[file]

	def id_to_file(self, identifier):
		"""Returns a filename given an identifier"""
		return self._id_to_file[identifier]

	def get_identifiers(self):
		"""Returns list of submission identifiers"""
		return [file["identifier"] for file in self._metadata]

	def get_filenames(self):
		"""Returns list of filenames in the submission directory"""
		return [file["filename"] for file in self._metadata]

class CanvasParser:
	"""Metadata parser for Canvas exports"""
	def __init__(self, submissions_dir):
		# list all files in the submissions directory since Canvas
		# doesn't output a metadata file
		submissions = list_files(submissions_dir)

		# intialize metadata list and iterate through the files
		self._metadata = []
		for file in submissions:
			# extract the identifier from the filename, which is everything
			# before the first underscore, meaning
			# somestudent_12345_67890_SomeStudent.ipynb -> somestudent
			identifier = re.match(r"(\w+?)\_", file)[1]

			# add the identifier and filename to the metadata list
			self._metadata += [{
				"identifier": identifier,
				"filename": file
			}]

		self._file_to_id = {file["filename"] : file["identifier"] for file in self._metadata}
		self._id_to_file = {file["identifier"] : file["filename"] for file in self._metadata}

	def get_metadata(self):
		"""Returns mapping of identifiers to files"""
		return self._metadata

	def file_to_id(self, file):
		"""Returns a identifier given a filename"""
		return self._file_to_id[file]

	def id_to_file(self, identifier):
		"""Returns a filename given an identifier"""
		return self._id_to_file[identifier]

	def get_identifiers(self):
		"""Returns list of submission identifiers"""
		return [file["identifier"] for file in self._metadata]

	def get_filenames(self):
		"""Returns list of filenames in the submission directory"""
		return [file["filename"] for file in self._metadata]


class JSONParser:
	"""Metadata parser for JSON format"""
	def __init__(self, file_path):
		# open JSON file and load contents into a string
		with open(file_path) as f:
			metdata = f.read()

		# parse string with json library
		self._metadata = json.loads(metdata, parse_int=str)

		# check that the JSON file has the correct structure
		assert type(self._metadata) == list, "JSON metadata is not a list"
		
		# check that each item in the JSON file is of correct type
		# and has "identifier" and "filename" keys
		for file in self._metadata:
			assert type(file) == dict, "JSON metadata does not have dictionaries"
			assert "identifier" in file.keys(), "JSON metadata does not contain \"identifier\" key"
			assert "filename" in file.keys(), "JSON metadata does not contain \"filename\" key"

		self._file_to_id = {file["filename"] : file["identifier"] for file in self._metadata}
		self._id_to_file = {file["identifier"] : file["filename"] for file in self._metadata}

	def get_metadata(self):
		"""Returns mapping of identifiers to files"""
		return self._metadata

	def file_to_id(self, file):
		"""Returns a identifier given a filename"""
		return self._file_to_id[file]

	def id_to_file(self, identifier):
		"""Returns a filename given an identifier"""
		return self._id_to_file[identifier]

	def get_identifiers(self):
		"""Returns list of submission identifiers"""
		return [file["identifier"] for file in self._metadata]

	def get_filenames(self):
		"""Returns list of filenames in the submission directory"""
		return [file["filename"] for file in self._metadata]

class YAMLParser:
	"""Metadata parser for YAML format"""
	def __init__(self, file_path):
		# open the YAML file and parse with yaml library
		with open(file_path) as f:
			self._metadata = yaml.safe_load(f)

		# convert any integer identifiers to strings
		for file in self._metadata:
			file["identifier"] = str(file["identifier"])

		# check that the YAML file has the correct structure
		assert type(self._metadata) == list, "YAML metadata is not a list"
		
		# check that each item in the YAML file is of correct type
		# and as "identifier" and "filename" keys
		for file in self._metadata:
			assert type(file) == dict, "YAML metadata does not have dictionaries"
			assert "identifier" in file.keys(), "YAML metadata does not contain \"identifier\" key"
			assert "filename" in file.keys(), "YAML metadata does not contain \"filename\" key"

		self._file_to_id = {file["filename"] : file["identifier"] for file in self._metadata}
		self._id_to_file = {file["identifier"] : file["filename"] for file in self._metadata}

	def get_metadata(self):
		"""Returns mapping of identifiers to files"""
		return self._metadata

	def file_to_id(self, file):
		"""Returns a identifier given a filename"""
		return self._file_to_id[file]

	def id_to_file(self, identifier):
		"""Returns a filename given an identifier"""
		return self._id_to_file[identifier]

	def get_identifiers(self):
		"""Returns list of submission identifiers"""
		return [file["identifier"] for file in self._metadata]

	def get_filenames(self):
		"""Returns list of filenames in the submission directory"""
		return [file["filename"] for file in self._metadata]